unit ChipmunkRenderDef;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Forms,
  Dialogs, ExtCtrls, GLBitmapFont, GLWindowsFont, GLCadencer, GLWin32Viewer,
  GLMaterial, GLScene, GLObjects, GLCoordinates, GLCrossPlatform, BaseClasses,
  GLColor, jpeg, tga, glKeyboard, vectorTypes, VectorGeometry,
  Controls, pngimage, GLRenderContextInfo,
  GLGraphics, OpenGL1x, ChipmunkImport;

procedure DrawPhysics(Sender: TObject; var rci: TRenderContextInfo;
                      ASpace : pcpSpace;
                      ShowBodies : boolean = true;
                      ShowShapes : boolean = true;
                      ShowConstraints : boolean = true;
                      ShowContacts : boolean = true
                      );


implementation


procedure drawcircle(x,y,r,a : single);
var n : integer;
    rads,coef:cpFloat;
begin
  glColor3f(0.0, 1.0, 0.0);
//  glLineWidth(4);

  coef := 2.0*M_PI/15;
  glBegin(GL_LINE_STRIP);
  for n := 0 to 15 do begin
     rads := n*coef;
     glVertex3f(r*cos(rads + a) + x, r*sin(rads + a) + y,0);
  end;
  glVertex3f(x,y,0);
  glEnd();
end;

procedure drawCircleShape(shape : pcpshape);
var body:pcpBody;
    c:cpVect;
    circle:pcpCircleShape;
begin
   body:=shape.body;
   circle := pointer(shape);
   c := circle.tc;
   drawCircle(c.x, c.y, circle.r, body.a);
end;

procedure drawSegmentShape(shape : pcpshape);
var seg:pcpSegmentShape;
    a,b:cpVect;
//    p:cpVect;
begin
   seg:=pointer(shape);
   a := seg.ta;
   b := seg.tb;

   glColor3f(1.0, 1.0, 1.0);
   if seg.r<>1 then glLineWidth(seg.r+1);

   glBegin(GL_LINES);
   	glVertex3f(a.x, a.y,0);
   	glVertex3f(b.x, b.y,0);
      // show normal
//      p:=cpvLerp(a,b,0.5);
//      glVertex3f(p.x, p.y,0);
//      p:=cpvAdd(p,cpvMult(seg.tn,0.1));
//      glVertex3f(p.x, p.y,0);

   glEnd();
   glLineWidth(2);
end;

procedure drawbodies(obj : pointer; data : pointer);
var b : pcpbody;
begin
   b:=obj;
   glColor3f(1.0, 1.0, 0.0);
   glBegin(GL_POINTS);
      glVertex3f(b.p.x, b.p.y,0);
   glEnd();
end;

procedure drawConstraints(obj : pointer; data : pointer);
var constraint : pcpconstraint;
    spring:pcpdampedSpring;
    pinjoint:pcpPinJoint;
    pivotjoint:pcpPivotJoint;
    a,b:cpVect;
begin
   constraint:=obj;
   if constraint.klass.cpType=CP_BREAKABLE_JOINT then begin
      constraint:=pcpBreakableJoint(obj).delegate;
      if not assigned(constraint) then exit;
   end;

  case constraint.klass.cpType of
   CP_SPRING_JOINT:begin
      spring:=pointer(constraint);

      a := cpvadd(constraint.a.p,spring.r1);
      b := cpvadd(constraint.b.p,spring.r2);

      glColor3f(1.0, 1.0, 1.0);
      glBegin(GL_LINES);
      	glVertex3f(a.x, a.y,0);
   	   glVertex3f(b.x, b.y,0);
      glEnd();
   end;
   CP_PIN_JOINT:begin
      pinjoint:=pointer(constraint);

		a := cpvadd(constraint.a.p, pinjoint.r1);
		b := cpvadd(constraint.b.p, pinjoint.r2);

      glColor3f(1.0, 0.0, 1.0);
      glPointSize(8.0);
		glBegin(GL_POINTS);
			glVertex3f(a.x, a.y,0);
			glVertex3f(b.x, b.y,0);
		glEnd();
      glPointSize(4.0);
		glBegin(GL_LINES);
			glVertex3f(a.x, a.y,0);
			glVertex3f(b.x, b.y,0);
		glEnd();
   end;
   CP_PIVOT_JOINT:begin
      pivotjoint:=pointer(constraint);

		a := cpvadd(constraint.a.p, pivotjoint.r1);
		b := cpvadd(constraint.b.p, pivotjoint.r2);

      glColor3f(0.0, 1.0, 1.0);
		glPointSize(8.0);
		glBegin(GL_POINTS);
			glVertex3f(a.x, a.y,0);
			glVertex3f(b.x, b.y,0);
		glEnd();
		glPointSize(4.0);
		glBegin(GL_LINES);
			glVertex3f(a.x, a.y,0);
			glVertex3f(b.x, b.y,0);
		glEnd();
   end;
  end;
end;

procedure drawPolyShape(shape : pcpshape);
var poly : PcpPolyShape;
    num,i : Integer;
    v:cpVect;
    verts:PcpVectArray;
begin
   poly:=pointer(shape);
   num:=poly.numVerts;
   verts := poly.tverts;

   glColor3f(0.0, 0.0, 1.0);
   glBegin(GL_LINE_LOOP);
   for i:=0 to num-1 do begin
      v := verts[i];
  		glVertex3f(v.x, v.y,0);
   end;
   glEnd();

   glBegin(GL_LINES);
      v:=shape.body.p;
   	glVertex3f(v.x, v.y,0);
      v:=cpvadd(v,cpvMult(shape.body.rot,(shape.bb.r-shape.bb.l)*0.25));
   	glVertex3f(v.x, v.y,0);
   glEnd();
end;

procedure drawShapes(obj : pointer; data : pointer);
var shape : pcpshape;
begin
   shape:=obj;
   case shape.klass.cptype of
      CP_CIRCLE_SHAPE:drawCircleShape(shape);
      CP_SEGMENT_SHAPE:drawSegmentShape(shape);
      CP_POLY_SHAPE:drawPolyShape(shape);
   end;
end;

procedure DrawCollisions(ptr : pointer; data : pointer);
var arb : pcpArbiter;
    i : integer;
    v:cpVect;
begin
   arb:=ptr;
   glColor3f(1.0, 0.0, 0.0);
   glBegin(GL_POINTS);
   for i:=0 to arb.numContacts-1 do begin
	  	v := arb.contacts[i].p;
	  	glVertex3f(v.x, v.y,0);
	 end;
   glEnd();
end;

procedure DrawPhysics(Sender: TObject; var rci: TRenderContextInfo;
                      ASpace : pcpSpace;
                      ShowBodies : boolean = true;
                      ShowShapes : boolean = true;
                      ShowConstraints : boolean = true;
                      ShowContacts : boolean = true
                      );

begin
   if not assigned(Space) then exit;
   glPushAttrib(GL_ALL_ATTRIB_BITS);
   try
      glEnable(GL_LINE_SMOOTH);
      glEnable(GL_POINT_SMOOTH);
      glDisable(GL_LIGHTING);
      glDisable(GL_DEPTH_TEST);
      glLineWidth(2);
      glPointSize(6);
      try
         if ShowConstraints then cpArrayEach(ASpace.constraints, drawConstraints, nil);
         if ShowShapes then begin
            cpSpaceHashEach(ASpace.activeShapes,drawshapes,nil);
            cpSpaceHashEach(ASpace.staticShapes,drawshapes,nil);
         end;
         if ShowBodies then cpArrayEach(ASpace.bodies, drawBodies, nil);
         if ShowContacts then cpArrayEach(ASpace.arbiters, DrawCollisions, nil);
      except
         exit;
      end;
   finally
      glPopAttrib;
   end;
end;

end.
