unit ViewDef;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Forms, math,
  Dialogs, StdCtrls, GLScene, GLObjects, GLMisc, GLWin32Viewer, GLBitmapFont,
  GLWindowsFont, GLTexture, GLCadencer, VectorTypes, VectorGeometry, GLKeyboard,
  GLGeomObjects,ChipmunkImport,OpenGL1x,VectorLists, ExtCtrls, GLHUDObjects,
  GLMaterial, GLCoordinates, GLCrossPlatform, BaseClasses, GLRenderContextInfo,
  Controls;

// chipmunk demo by Paul Robello
// paulr@par-comnet

type
TVirtArray = array of cpVect; // used to dynamicly create and change vertex data

TPenMode = (pmFirst,pmMotor,pmMove,pmCircle,pmPoly,pmSegments,pmChain,pmChainBreakable,pmPin,pmPivot,pmLast); // only poly,segments and chain are implemnted
TPenBody = (pbStatic,pbDynamic,pbDelete); // only static and dynamic are implemented

const // these are just used for display
TPenModeSt :array[pmFirst..pmLast] of string = ('First','Motor','Move','Circle','Poly','Segments','Chain','Breakable Chain','Pin','Pivot','Last');
TPenBodySt :array[pbStatic..pbDelete] of string = ('Static','Dynamic','Delete');
SimStepSize =0.001;

type
  TForm1 = class(TForm)
    GLScene1: TGLScene;
    GLCadencer1: TGLCadencer;
    GLMaterialLibrary1: TGLMaterialLibrary;
    GLWindowsBitmapFont1: TGLWindowsBitmapFont;
    GLSceneViewer1: TGLSceneViewer;
    GLCamera1: TGLCamera;
    World: TGLDummyCube;
    GLDirectOpenGL1: TGLDirectOpenGL;
    FPSTimer: TTimer;
    GLHUDText1: TGLHUDText;
    procedure GLSceneViewer1MouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure FPSTimerTimer(Sender: TObject);
    procedure GLDirectOpenGL1Render(Sender: TObject;
      var rci: TRenderContextInfo);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure GLSceneViewer1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure GLSceneViewer1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure GLCadencer1Progress(Sender: TObject; const deltaTime,
      newTime: Double);
  private
    { Private declarations }
  public
    { Public declarations }
    mpv : TAffineVector;
    spline : TGLLInes; // used for drawing things
    mousePoint_last : cpVect;
    function GetScreenVec: TAffineVector;
    function mouseToSpace(x,y:integer):TAffineVector;

    procedure SetupWalls(asize : single = 10); // setup 3 static walls to contain shapes (does not resise with window
    procedure StartSpline; // starts a new drawing
    procedure EndSpline; // ends drawing and adds to sim
    procedure AddChainToSim(ALine : TGLLines); // adds a chain to the sim
    function AddPolyToSim(Poly : TGLLines; checkshape : boolean = true) : pcpshape; // adds a true poly or segment poly to sim
    function AddRectangleToSim(x,y,width,height : single):pcpshape; // adds a rectangle to sim
    function AddSegmentToSim(x,y,width,radius : single):pcpshape; // adds a single segment to sim
    function AddCircleToSim(const r : cpFloat; const center : cpVect) : pcpshape; // adds a circle to sim
    function AddMotorToSim(const r : cpFloat; const center : cpVect) : pcpshape; // adds a circle to sim

    procedure ClearAll;
    procedure ResetScene;
    procedure GrabShape;
    procedure ReleaseShape;
    procedure PinShapes;
  end;

var
  Form1: TForm1;
  Pause : boolean = false; // pause simulation
  PenMode  : TPenMode = pmPoly;
  PenBody : TPenBody = pbDynamic;
  GroupNum : integer = 1; // used to group shapes
  segSize:single = 0.01; // distance between nodes in a drawing before a new one is added
  ShowBodies : boolean = true; // show body aka center of mass for each shape group
  ShowCollisions : boolean = true; // show the contact points for collisions
  ShowConstraints : boolean = true; // show springs
  mouseJoint : pcpPivotJoint;
  mouseBody : pcpBody;


implementation

uses ChipmunkRenderDef;

{$R *.dfm}

Var
   StaticBody: PcpBody; // all static shapes use this

function VectToCPV(const aVect : TAffineVector) : cpvect; inline;
begin
   result.x:=AVect[0];
   result.y:=AVect[1];
end;

// tests a vertex list for convexity
function IsConvex(var points: TVirtArray; normal : cpFloat = -1) : boolean;
var vi,vj : cpvect;
    n : cpFloat;
    i,j : integer;
begin
   result:=False; // assume not convex
   if length(points)<3 then exit; // not enough points to be convex
   vi := cpvsub(points[1],points[0]);
   for i:= 1 to high(points) do begin
      j  := (i+1) mod length(points);
      vj := cpvsub(points[j],points[i]);
      n:=cpvcross(vi,vj);

      if ((n*normal) < 0.0) then exit; // if norm is <0 then poly is concave
      vi:=vj;
   end;
   result:=True;
end;

// this uses same logic as isConvex but removes offending verts and returns number of verts removed
function MakeConvex(var verts: TVirtArray; nverts : integer; normal : cpFloat = -1) : integer;
var vi,vj : cpvect;
    n : cpFloat;
    i,j,k : integer;
    found : boolean;
begin
   result:=0;
   repeat
      found:=False;
      if nverts<4 then exit;
      setlength(verts,nverts);

      vi := cpvsub(verts[1],verts[0]);
      i:=1;
      while i<nverts do begin
         j  := (i+1) mod nverts;
         vj := cpvsub(verts[j],verts[i]);
         n:=cpvcross(vi,vj);

         if ((n*normal) < 0.0) then begin
            inc(result);
            for k:=i to nverts-2 do verts[k]:=verts[k+1];
            dec(nverts);
            found:=True;
            break;
         end;
         vi:=vj;
         inc(i);
      end;
   until not found;
end;

function PolyArea(var verts:TVirtArray) : cpFloat;
var i : integer;
begin
   result:= ( verts[0].x * verts[high(verts)].y - verts[high(verts)].x * verts[0].y) * 0.5;
   for i:=1 to high(verts) do begin
      result:= result +abs( verts[i].x * verts[i-1].y - verts[i-1].x * verts[i].y) * 0.5;
   end;
end;

function PolyMass(var verts:TVirtArray; const density : cpFloat) : cpFloat;
begin
   result:=PolyArea(verts)*density;
end;

//this moves vertex points of poly into position for use with being merged with new/diffrent body
function cpPolyShapeShift( const poly : PcpPolyShape; offset,rot2 : cpVect) : PcpPolyShape;
var i : integer;
    a,b,n : cpVect;
begin
   result:=poly;

   for i:=0 to poly.numVerts-1 do begin
      a := poly.verts[i];
      b := poly.verts[(i+1) mod poly.numVerts];

      if not cpvEqual(rot2,cpvright) and (abs(cpvtoangle(rot2))>0.01) then begin // try to correct for inital collision
         a := cpvunrotate(a,rot2);
         if i<poly.numVerts-1 then b := cpvunrotate(b,rot2);
      end;

      a := cpvadd(a,offset);
      if i<poly.numVerts-1 then b := cpvadd(b,offset);

      n := cpvnormalize(cpvperp(cpvsub(b, a)));

      poly.verts[i] := a;
      poly.axes[i].n := n;
      poly.axes[i].d := cpvdot(n, a);
   end;
end;

//this moves vertex points into position for use with being merged with new/diffrent body
function cpSegmentShapeShift( seg : PcpSegmentShape; const offset,rot2 : cpVect) : PcpSegmentShape;
begin
   result:=seg;

   if not cpvEqual(rot2,cpvRight) and (abs(cpvtoangle(rot2))>0.01) then begin // try to correct for inital collision
      seg.a:=cpvUnRotate(seg.a,rot2);
      seg.b:=cpvUnRotate(seg.b,rot2);
   end;

   seg.a:=cpvadd(seg.a,offset);
   seg.b:=cpvadd(seg.b,offset);
end;

function cpCircleShapeShift( circ : PcpCircleShape; const offset,rot2 : cpVect) : PcpCircleShape;
begin
   result:=circ;
   if not cpvEqual(rot2,cpvRight) and (abs(cpvtoangle(rot2))>0.01) then circ.c:=cpvunrotate(circ.c,rot2); // try to correct for inital collision
   circ.c:=cpvadd(circ.c,offset);
end;

function cpShapeShift( const shape : PcpShape; offset,rot2 : cpVect) : PcpShape;
begin
   result:=shape;
   case shape.klass.cptype of
      CP_CIRCLE_SHAPE: cpCircleShapeShift(pointer(shape),offset,rot2);
      CP_SEGMENT_SHAPE: cpSegmentShapeShift(pointer(shape),offset,rot2);
      CP_POLY_SHAPE: cpPolyShapeShift(pointer(shape),offset,rot2);
   end;
end;

// get the center of mass for each type of shape
function cpShapeCenterOfMass( shape : pcpShape) : cpVect;
var seg : PcpSegmentShape;
    poly : PcpPolyShape;
    i : integer;
begin
   result:=cpvZero;
   if shape.klass.cptype=CP_SEGMENT_SHAPE then begin
      seg:=pcpSegmentShape(shape);
      result:=cpvLerp(seg.a,seg.b,0.5);
   end else if shape.klass.cptype=CP_CIRCLE_SHAPE then begin
      result:=pcpCircleShape(shape).c;
   end else if shape.klass.cptype=CP_POLY_SHAPE then begin
      poly:=PcpPolyShape(shape);
      for i:=0 to poly.numVerts-1 do result:=cpvadd(result,poly.verts[i]);
      result:=cpvmult(result,1/poly.numVerts);
   end;
end;

function cpBodyCenterOfMass( body : pcpbody) : cpVect;
var i : integer;
begin
   result:=cpvzero;
   for i:=0 to body.shapes.num-1 do
      result:=cpvAdd(result,cpShapeCenterOfMass(body.shapes.arr[i]));
   result:=cpvMult(result,1/body.shapes.num);
end;

procedure AddPivot(ba,bb : pcpBody; const pivot : cpVect);
begin
   cpSpaceAddConstraint(space,cpPivotJointNew(ba,bb,pivot));
end;

procedure PinNewCollisions(ptr : pointer; data : pointer);
var arb : pcpArbiter;
    joint : pcpConstraint;
    sa,sb : pcpshape;
    ba,bb : pcpBody;
    aa,ab : cpVect;
    i : integer;
begin
   arb:=ptr;
   sa:=arb.a;
   sb:=arb.b;
   ba:=sa.body;
   bb:=sb.body;

   // if nether shape is a chain then bail
   if not (sa.tag in [1..3]) and not (sb.tag in [1..3]) then exit;

   if ba=bb then exit; // same body should never hapen
   if (ba.m=infinity) and (bb.m=infinity) then exit; // both static no joint needed

   if (sa.collision_type<>1) and (sb.collision_type<>1) then exit; // nether is new

   if ba.m=infinity then aa:=bb.p
      else aa:=cpvSub(bb.p,ba.p);

   if bb.m=infinity then ab:=ba.p
      else ab:=cpvSub(ba.p,bb.p);

   for i:=0 to arb.numContacts-1 do begin
      joint:=cpPinJointNew(ba,bb,arb.contacts[i].p);
      cpSpaceAddConstraint(space,joint);
   end;

   if (sa.collision_type=1) then begin
      sa.group:=sb.group;
   end else begin
      sb.group:=sa.group;
   end;
   // make sure merge shapes does not try anything
   sa.collision_type:=0;
   sb.collision_type:=0;
end;

procedure MergeNewCollisions(ptr : pointer; data : pointer);
var arb : pcpArbiter;
    sa,sb : pcpshape;
    ba,bb : pcpBody;
    aa,ab : cpVect;
    i,j,n : integer;
begin
   arb:=ptr;
   if arb.numContacts<1 then exit; // no contacts should never happen
   sa:=arb.a;
   sb:=arb.b;
   ba:=sa.body;
   bb:=sb.body;

   if ba=bb then exit; // same body should never hapen
   if (ba.m=infinity) and (bb.m=infinity) then exit; // both static no merge needed

   if (sa.collision_type<>1) and (sb.collision_type<>1) then exit; // nether is new bail

   if ba.m=infinity then aa:=bb.p
      else aa:=cpvSub(bb.p,ba.p);

   if bb.m=infinity then ab:=ba.p
      else ab:=cpvSub(ba.p,bb.p);

   // half way between the shapes
   aa:=cpvMult(aa,0.5);
   ab:=cpvMult(ab,0.5);

   if (sa.collision_type=1) then begin
      n:=ba.shapes.num; // save number of shapes transferd to new body
      for i:=0 to ba.shapes.num-1 do begin // add shapes to new body and set group
         cpArrayPush(bb.shapes,ba.shapes.arr[i]);
         pcpShape(ba.shapes.arr[i]).body:=bb;
         pcpShape(ba.shapes.arr[i]).group:=sb.group;
      end;
      // add part of mass and moment to new body
      cpBodySetMass(bb,bb.m+ba.m*0.5);
      cpBodySetMoment(bb,bb.i+ba.i*0.5);

      // remove and free old body
      cpSpaceRemoveBody(space,ba);
      cpBodyFree(ba);

      bb.p:=cpvadd(bb.p,ab);// move body half way twards other body

      //convert offsets to local
      aa:=cpvunrotate(aa,bb.rot);
      ab:=cpvunrotate(ab,bb.rot);

      for i:=bb.shapes.num-n to bb.shapes.num-1 do // shift existing shapes to compensate for new body pos
         cpShapeShift(bb.shapes.arr[i],ab,bb.rot); // move shape half way to new body

      for i:=0 to bb.shapes.num-(n+1) do // shift existing shapes to compensate for new body pos
         cpShapeShift(bb.shapes.arr[i],aa,cpvright);

      for j:=0 to 3 do begin
      // find new corect center of mass and apply to body and all shapes
      aa:=cpvmult(cpBodyCenterOfMass(bb),1.0);
      bb.p:=cpvadd(bb.p,aa);
      ab:=cpvMult(aa,-1);

      // convert offset to local and shift all shapes
      ab:=cpvunrotate(ab,bb.rot);
      for i:=0 to bb.shapes.num-1 do
         cpShapeShift(bb.shapes.arr[i],ab,cpvright);
      end;

   end else begin
      n:=bb.shapes.num;
      for i:=0 to bb.shapes.num-1 do begin
         cpArrayPush(ba.shapes,bb.shapes.arr[i]);
         pcpShape(bb.shapes.arr[i]).body:=ba;
         pcpShape(bb.shapes.arr[i]).group:=sa.group;
      end;
      cpBodySetMass(ba,ba.m+bb.m*0.5);
      cpBodySetMoment(ba,ba.i+bb.i*0.5);

      cpSpaceRemoveBody(space,bb);
      cpBodyFree(bb);

      ba.p:=cpvadd(ba.p,aa);

      aa:=cpvunrotate(aa,ba.rot);
      ab:=cpvunrotate(ab,ba.rot);

      for i:=ba.shapes.num-n to ba.shapes.num-1 do
         cpShapeShift(ba.shapes.arr[i],aa,ba.rot);

      for i:=0 to ba.shapes.num-(n+1) do
         cpShapeShift(ba.shapes.arr[i],ab,cpvright);

      for j:=0 to 3 do begin
      aa:=cpvMult(cpBodyCenterOfMass(ba),1.0);
      ba.p:=cpvadd(ba.p,aa);
      ab:=cpvMult(aa,-1);

      ab:=cpvunrotate(ab,ba.rot);
      for i:=0 to ba.shapes.num-1 do
         cpShapeShift(ba.shapes.arr[i],ab,cpvright);
      end;
   end;
end;


// after one pass a shape is not considerd new and wont merge with existing shapes
procedure updateShapes(obj : pointer; data : pointer);
var shape : pcpshape;
begin
   if pause then exit;
   shape:=obj;
   shape.collision_type:=0;
end;

// static walls to contain created shapes
procedure TForm1.SetupWalls(asize : single = 10);
var f : pcpShape;
begin
   //add ceiling
   f:=cpSegmentShapeNew(StaticBody,cpv(asize,asize),cpv(-asize,asize),0);
   f.e:=0.25;
   f.u:=0.75;
//   f.group:=GooStaticShapeGroup;
   cpSpaceAddStaticShape(space,f);

   //add floor
   f:=cpSegmentShapeNew(StaticBody,cpv(-asize,0),cpv(asize,0),0);
   f.e:=0.25;
   f.u:=0.75;
//   f.group:=GooStaticShapeGroup;
   cpSpaceAddStaticShape(space,f);

   //left wall
   f:=cpSegmentShapeNew(StaticBody,cpv(-asize,asize),cpv(-asize,0),0);
   f.e:=0.25;
   f.u:=0.75;
//   f.group:=GooStaticShapeGroup;
   cpSpaceAddStaticShape(space,f);

   //right wall
   f:=cpSegmentShapeNew(StaticBody,cpv(asize,0),cpv(asize,asize),0);
   f.e:=0.25;
   f.u:=0.75;
//   f.group:=GooStaticShapeGroup;
   cpSpaceAddStaticShape(space,f);
end;

function TForm1.mouseToSpace(x,y:integer):TAffineVector;
var mx, my, mz:GLdouble;
    model,proj:THomogeneousDblMatrix;
    view:THomogeneousIntVector;
begin
   setmatrix(proj,GLSceneViewer1.Buffer.ProjectionMatrix);
   setmatrix(model,GLSceneViewer1.Buffer.ModelViewMatrix);
   view:=PHomogeneousIntVector(@GLSceneViewer1.Buffer.ViewPort)^;
//	glGetDoublev(GL_MODELVIEW_MATRIX, @model);
//	glGetDoublev(GL_PROJECTION_MATRIX, @proj);
//	glGetIntegerv(GL_VIEWPORT, @view);
	gluUnProject(x, GLSceneViewer1.Height - y, 0.0, model, proj, view, @mx, @my, @mz);

	result:=AffineVectorMake(mx, my, mz);
end;

function TForm1.GetScreenVec: TAffineVector;
var mp : tpoint;
begin
   GetCursorPos(mp);
   mp:=GLSceneViewer1.ScreenToClient(mp);
   result:=MouseToSpace(mp.x,mp.y);
   result[2]:=0;
end;

procedure TForm1.GLCadencer1Progress(Sender: TObject; const deltaTime, newTime: Double);
var s : integer;
begin
   GLSceneViewer1.Invalidate; // force redraw and animation

   if not pause then begin
      for s:=0 to floor(deltaTime/SimStepSize) do cpSpaceStep(Space, SimStepSize); // multiple steps per frame gives more stable look
   end;

   // if new shape overlaps existing shape pin them together
   cpArrayEach(space.arbiters, PinNewCollisions, nil);
   cpArrayEach(space.arbiters, MergeNewCollisions, nil);

   // make new shapes as old so they dont continue to pin to other shapes
   cpSpaceHashEach(space.activeShapes,updateshapes,nil);
   cpSpaceHashEach(space.staticShapes,updateshapes,nil);

   if active and isKeyDown(VK_LBUTTON) then begin
      mpv:=getScreenVec;
      if assigned(mousebody) then begin
         mouseBody.p := VectToCpv(mpv);
         mouseBody.v := cpvmult(cpvsub(mousePoint_last,mouseBody.p), 60.0);
         mousePoint_last := mouseBody.p;
      end;

      if penMode<>pmMove then begin
         if not assigned(spline) then StartSpline;
         if spline.Nodes.Count=0 then begin
            spline.Nodes.Add.AsAffineVector:=mpv; // starting point
            spline.Nodes.Add.AsAffineVector:=mpv; // current point
         end;

         // if nodes are far enough appart add a new one else update existing one
         if VectorDistance2(spline.Nodes[spline.Nodes.Count-1].AsAffineVector,spline.Nodes[spline.Nodes.Count-2].AsAffineVector)>segSize
            then spline.Nodes.Add.AsAffineVector:=mpv
               else spline.Nodes[spline.Nodes.Count-1].AsAffineVector:=mpv;
      end;
   end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
   cpInitChipmunk;
   cpResetShapeIdCounter; // not realy needed but makes for more complete demo

   Space:= cpSpaceNew;
   cpSpaceResizeStaticHash(Space, 0.1, 500);
   cpSpaceResizeActiveHash(Space, 0.1, 999);

   Space.gravity:= cpv(0, -9.81);
   Space.iterations:=6;
   space.damping:=0.1;

   StaticBody:= cpBodyNew(INFINITY, INFINITY);

	if not assigned(mouseBody) then mousebody := cpBodyNew(INFINITY, INFINITY);


   SetupWalls; // setup container for our shapes
//   TestSpring;


   GLHudText1.Visible:=true;
   GLHudText1.Text:='? = show/hide this'+#13;
   GLHudText1.Text:=GLHudText1.Text+'Space = un/pause sim'+#13;
   GLHudText1.Text:=GLHudText1.Text+'C=Clear Sim'+#13;
   GLHudText1.Text:=GLHudText1.Text+'S=Static'+#13;
   GLHudText1.Text:=GLHudText1.Text+'D=Dynamic'+#13;
   GLHudText1.Text:=GLHudText1.Text+'M=Move shape'+#13;
   GLHudText1.Text:=GLHudText1.Text+'1=Poly'+#13;
   GLHudText1.Text:=GLHudText1.Text+'2=Segments'+#13;
   GLHudText1.Text:=GLHudText1.Text+'3=Chain'+#13;
   GLHudText1.Text:=GLHudText1.Text+'4=Breakable Chain'+#13;
   GLHudText1.Text:=GLHudText1.Text+'5=Right click creates Circle'+#13;
   GLHudText1.Text:=GLHudText1.Text+'6=Right click creates Motor'+#13;
   GLHudText1.Text:=GLHudText1.Text+'9=Right click creates Pins Shapes (they must overlap)'+#13;

   GLHudText1.Text:=GLHudText1.Text+'Right Click = Add Rectangle'+#13;
   GLHudText1.Text:=GLHudText1.Text+'Left Drag = Draw Shape'+#13;
   GLHudText1.Text:=GLHudText1.Text+'New Shapes touching another shape will be pined to other shape'+#13;
   GLHudText1.Text:=GLHudText1.Text+'When in poly mode shapes that are nearly convex will be made convex'+#13;
   GLHudText1.Text:=GLHudText1.Text+'When in poly mode shapes close to rectangle and circle will be made perfect'+#13;
   GLHudText1.Text:=GLHudText1.Text+'Static Chains are pined at there starting point'+#13;
//   GLHudText1.Text:=GLHudText1.Text+'T=Show Test of motor and spring'+#13;
end;

procedure TForm1.ClearAll;
begin
   cpFreeArbiters(Space);
   cpSpaceFreeChildren(Space);
   cpSpaceFree(Space);
   cpBodyFree(staticBody);
   cpBodyFree(mouseBody);
   if assigned(mouseJoint) then cpConstraintFree(pointer(mouseJoint));
   mouseJoint:=nil;
   mouseBody:=nil;
   staticBody:=nil;
end;

procedure TForm1.ResetScene;
begin
   ClearAll;
   FormCreate(self);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
   ClearAll;
end;

type bodyarray=array[0..10] of pcpBody;

procedure findBody(shape:PcpShape; data:pointer);
var pb : ^bodyarray;
    i : integer;
begin
   pb:=data;
   for i:=0 to high(pb^) do begin
      if assigned(pb^[i]) then continue;
      pb^[i]:=shape.body;
      exit;
   end;
end;

procedure TForm1.GrabShape;
var body:bodyarray;
   point:cpVect;
begin
   zeromemory(@body[0],sizeof(body));
   point:=cpv(mpv[0],mpv[1]);
   cpSpaceShapePointQuery(space, point, findBody, @body[0]);
   if not assigned(body[0]) then exit;

   MouseBody.p:=point;
   mousePoint_last := mouseBody.p;
   mouseBody.v := cpvzero;
   cpBodyResetForces(mouseBody);

   mouseJoint := pcpPivotJoint(cpPivotJointNew(mouseBody, body[0], cpvzero, cpBodyWorld2Local(body[0], point)));
	mouseJoint.constraint.maxForce := 50000.0;
	mouseJoint.constraint.biasCoef := 0.15;
	cpSpaceAddConstraint(space, pcpConstraint(mouseJoint));
end;

procedure TForm1.PinShapes;
var body:bodyarray;
   point:cpVect;
begin
   zeromemory(@body[0],sizeof(body));
   point:=cpv(mpv[0],mpv[1]);
   cpSpaceShapePointQuery(space, point, findBody, @body[0]);
   if not (assigned(body[0]) or assigned(body[1])) then exit;

//   cpSpaceAddConstraint(space, cpPivotJointNew(body[0], body[1], cpvzero,cpBodyWorld2Local(body[0], point)));

   if (assigned(body[0]) and assigned(body[1])) then begin
      cpSpaceAddConstraint(space, cpPivotJointNew(body[0], body[1], point));
      pcpShape(body[1].shapes.arr[0]).group:=pcpShape(body[0].shapes.arr[0]).group;
   end else begin
      cpSpaceAddConstraint(space, cpPivotJointNew(body[0], StaticBody, point));
   end;
end;

procedure TForm1.ReleaseShape;
begin
   if not assigned(mouseJoint) then exit;
   cpSpaceRemoveConstraint(space, pcpConstraint(mouseJoint));
	 cpConstraintFree(pcpConstraint(mouseJoint));
   mouseJoint := nil;
end;

procedure TForm1.GLSceneViewer1MouseDown(Sender: TObject; Button: TMouseButton;Shift: TShiftState; X, Y: Integer);
begin
   mpv:=GetScreenVec;
   if mbleft=button then begin
      if penMode=pmMove then grabShape else StartSpline;
   end;
   if mbright=button then begin
      case penMode of
         pmCircle:AddCircleToSim(0.1,cpv(mpv[0],mpv[1]));
         pmMotor:AddMotorToSim(0.1,cpv(mpv[0],mpv[1]));
         pmPin:PinShapes;
         else AddRectangleToSim(mpv[0],mpv[1],0.2,0.2);
      end;
   end;
end;

procedure TForm1.GLSceneViewer1MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   mpv:=getScreenVec;
   if mbleft=button then begin
      if penMode=pmMove then ReleaseShape else EndSpline;
   end;
end;

procedure TForm1.StartSpline;
begin
   if assigned(Spline) then begin
      spline.nodes.clear;
      exit;
   end;
   spline := TGLLines(World.addNewChild(TGLLines));
   spline.Translate(0,0,1); // make spline show in front of other shapes
   spline.LineWidth:=4;
   spline.AntiAliased:=True;
   spline.NodeColor.SetColor(1,0,0);
   spline.Material.MaterialOptions:=[moNoLighting];
//   spline.NodesAspect:=lnaInvisible;
   spline.NodesAspect:=lnaDodecahedron; // not needed can be used for debug
   spline.NodeSize:=0.05; // not needed can be used for debug
   spline.SplineMode:=lsmLines; // we use segment shapes so lines is more accurate
   spline.LinePattern:=$F0F0; // dashed line
end;

function TForm1.AddRectangleToSim(x,y,width,height : single) : pcpShape;
var i : integer;
    w,h : single;
    poly : TGLLines;
begin
   w:=width/2;
   h:=height/2;

   poly := TGLLines(World.addNewChild(TGLLines));
   poly.Translate(0,0,1);
   poly.LineWidth:=4;
   poly.NodesAspect:=lnaInvisible;
   poly.NodeSize:=0.05;
   poly.SplineMode:=lsmLines;
   poly.LinePattern:=$F0F0;

   poly.Nodes.AddNode(-w,h,0);
   poly.Nodes.AddNode(w,h,0);
   poly.Nodes.AddNode(w,-h,0);
   poly.Nodes.AddNode(-w,-h,0);

   for i:=0 to poly.nodes.count-1 do begin
      poly.nodes[i].x:=poly.nodes[i].x+x;
      poly.nodes[i].y:=poly.nodes[i].y+y;
   end;

   result:=AddPolyToSim(poly,false);
   poly.free;
end;

function TForm1.AddSegmentToSim(x,y,width,radius : single) : pcpshape;
var i : integer;
    Poly : TGLLines;
    w : single;
begin
   w:=width/2;
   Poly := TGLLines(world.addNewChild(TGLLines));
   Poly.Translate(0,0,1);
   Poly.Material.BlendingMode:=bmTransparency;
   Poly.Material.FrontProperties.Diffuse.SetColor(1,0,0,0.5);
   Poly.SplineMode:=lsmLines;

   Poly.Nodes.AddNode(-w,25,0);
   Poly.Nodes.AddNode(w,0,0);

   for i:=0 to Poly.nodes.count-1 do begin
      Poly.nodes[i].x:=Poly.nodes[i].x+x;
      Poly.nodes[i].y:=Poly.nodes[i].y+y;
   end;

   result:=AddPolyToSim(Poly,false);

   poly.free;
end;

function TForm1.AddCircleToSim(const r : cpFloat; const center : cpVect) : pcpshape;
var  body: pcpbody;
     shape: pcpshape;
     ca : cpFloat;
begin
   inc(groupnum);

   if PenBody=pbDynamic then begin
      ca:=cpCircleMass(r,1);
      body:= cpbodynew(ca, cpMomentForCircle(ca, r,0, cpvzero));
      body.p:=center;
      cpSpaceAddBody(space, body);
      shape:= cpCircleShapeNew(body, r, cpvZero);
   end else begin
      body:= StaticBody;
      shape:= cpCircleShapeNew(body, r, center);
   end;

   shape.e:=0.0; shape.u:= 1;
   shape.group:=groupnum;
   if body <> StaticBody then begin
      shape.collision_type:=1;
      cpSpaceAddShape(space, shape);
   end else begin
      cpSpaceAddStaticShape(space, shape);
   end;

   result:=shape;
end;

function TForm1.AddMotorToSim(const r : cpFloat; const center : cpVect) : pcpshape;
var  body: pcpbody;
     shape: pcpshape;
     ca : cpFloat;
     mj : pcpSimpleMotor;
begin
   inc(groupnum);

   if PenBody=pbDynamic then begin
      ca:=cpCircleMass(r,1);
      body:= cpbodynew(ca, cpMomentForCircle(ca, r,0, cpvzero));
      body.p:=center;
      cpSpaceAddBody(space, body);
      shape:= cpCircleShapeNew(body, r, cpvZero);
   end else begin
      body:= StaticBody;
      shape:= cpCircleShapeNew(body, r, center);
   end;

   shape.e:=0.0; shape.u:= 1;
   shape.group:=groupnum;
   if body <> StaticBody then begin
      shape.collision_type:=1;
      cpSpaceAddShape(space, shape);
   end else begin
      cpSpaceAddStaticShape(space, shape);
   end;

   mj := pcpSimpleMotor(cpSimpleMotorNew(StaticBody,shape.body,2));

   cpSpaceAddConstraint(space,pointer(mj));

   result:=shape;
end;

procedure TForm1.AddChainToSim(ALine: TGLLines);
var verts: TVirtArray;
    i : integer;
    body,body_last: pcpbody;
    shape: pcpshape;
    joint : pcpConstraint;
    bj : pcpConstraint;
begin
   with ALine do begin
      inc(groupnum);
      setlength(verts,nodes.count);
      for i:=0 to nodes.count-1 do verts[i]:=cpv(Nodes[i].X,Nodes[i].Y);

      body_last:=nil;
      for i:=0 to high(verts) do begin
         if (i=0) and (PenBody=pbStatic) then begin
            body:=StaticBody
         end else begin
            body:= cpbodynew(0.1, 0.1);
            body.p:=verts[i];
            cpSpaceAddBody(space, body);
            // bodies with tag 1 are dynamic starting body for chains
            // bodies with tag 2 are dynamic normal body for chains
            // bodies with tag 3 are dynamic end body for chains

            if i=0 then body.tag:=1 else body.tag:=2;
         end;

         if body<>staticBody then shape:=cpCircleShapeNew(body,0.01,cpvzero)
            else shape:=cpCircleShapeNew(body,0.01,verts[i]);
         shape.e:=0.0; shape.u:=0.01;
         shape.collision_type:=1;
         shape.tag:=1; // shapes with tag 1 are chain segments
         shape.group:=groupnum;
         cpSpaceAddShape(space, shape);
 
         if i>0 then begin
            if body_last=staticBody then
               joint:=cpDampedSpringNew(body_last,body,verts[i-1],cpvzero,cpvdist(verts[i-1],verts[i]),9000,0.5)
            else
               joint:=cpDampedSpringNew(body_last,body,cpvzero,cpvzero,cpvdist(verts[i-1],verts[i]),9000,0.5);

            if PenMode=pmChainBreakable then begin
               bj:=cpBreakableJointNew(joint,space,true);
               bj.maxForce:=0.124*nodes.count;
               cpSpaceAddConstraint(space,bj);
            end else begin
               cpSpaceAddConstraint(space,joint);
            end;
         end;
         body.data:=body_last;
         body_last:=body;
      end;
   end;
   if assigned(body_last) then body_last.tag:=3;
end;

//used for shape recognition
//creates a smaller and larger rectangle and checks if all points are within range
function isSqare(Poly :TGLLines; fuzz : single = 0.25) : boolean;
var i : integer;
    min,max,v : TAffineVector;
    w,h,w2,h2 : single;
begin
   result:=False;
   with Poly.nodes do begin
      GetExtents(min,max);
      w:=(max[0]-min[0])/2;
      h:=(max[1]-min[1])/2;
      w2:=w*(1+fuzz);
      h2:=h*(1+fuzz);
      w:=w*(1-fuzz);
      h:=h*(1-fuzz);

      for i:=0 to Count-1 do begin
         v:=Items[i].AsAffineVector;
         if not ((v[0]>-w2) and (v[0]<w2) and (v[1]>-h2) and (v[1]<h2)) then exit;
         if ((v[0]>-w) and (v[0]<w) and (v[1]>-h) and (v[1]<h)) then exit;
      end;
   end;
   result:=True;
end;

//used for shape recognition
//creates a smaller and larger circle and checks if all points are within range
function isCircle(Poly :TGLLines; fuzz : single = 0.25) : boolean;
var i : integer;
    min,max,v : TAffineVector;
    w,h,r1,r2,r : single;
begin
   result:=False;
   with Poly.nodes do begin
      GetExtents(min,max);
      w:=(max[0]-min[0])/2;
      h:=(max[1]-min[1])/2;
      if abs(w-h)>10+fuzz then exit;
      if w<h then r1:=w else r1:=h;
      r2:=r1*(1+fuzz);
      r1:=r1*(1-fuzz);
      r1:=r1*r1;
      r2:=r2*r2;

      for i:=0 to Count-1 do begin
         v:=Items[i].AsAffineVector;
         r:=VectorDistance2(v,NullVector);
         if not ((r>r1) and (r<r2)) then exit;
      end;
   end;
   result:=True;
end;

function TForm1.AddPolyToSim(Poly : TGLLines; checkshape : boolean = true) : pcpShape;
var verts: TVirtArray;
    i,j,r : integer;
    center,v : cpVect;
    body: pcpbody;
    shape: pcpshape;
    min,max : TAffineVector;
    pa,w,h : cpFloat;
    convex : boolean;
begin
   result:=nil;
   with Poly do begin
      nodes.GetExtents(min,max);
      w:=(max[0]-min[0])/2;
      h:=(max[1]-min[1])/2;
      center:=cpv(w+min[0],h+min[1]);
      setlength(verts,nodes.count);
      for i:=0 to nodes.count-1 do begin
         if PenBody=pbDynamic then begin
            verts[i]:=cpvsub(cpv(Nodes[i].X,Nodes[i].Y),center);
            Nodes[i].x:=Verts[i].x;
            Nodes[i].y:=Verts[i].y;
         end else begin
            verts[i]:=cpv(Nodes[i].X,Nodes[i].Y);
            v:=cpvsub(verts[i],center);
            Nodes[i].x:=v.x;
            Nodes[i].y:=v.y;
         end;
      end;
   end;
   if (penMode=pmPoly) then begin
      if checkshape then begin
         if isCircle(Poly) then begin
            caption:=caption+' circle';
            result:=AddCircleToSim(w,center);
            exit;
         end;
         if isSqare(Poly) then begin
            caption:=caption+' Rectangle';
            result:=AddRectangleToSim(center.x,center.y,w*2,h*2);
            exit;
         end;
      end;

      convex:=IsConvex(verts,-1);
      if not convex then begin
         r:=MakeConvex(verts,length(Verts));
         if r>length(Verts)*0.1 then begin
            setlength(verts,Poly.Nodes.count);
            for i:=0 to Poly.Nodes.count-1 do begin
               verts[i]:=cpv(Poly.Nodes[i].X,Poly.Nodes[i].Y);
               if PenBody<>pbDynamic then verts[i]:=cpvadd(verts[i],center);
            end;
         end;
         convex:=IsConvex(verts,-1);
      end;
   end else convex:=False;


   if PenBody=pbDynamic then begin
      pa:=Polymass(verts,1);
      body:= cpbodynew(pa, cpMomentForPoly(pa, length(verts), @verts[0], cpvzero));
      body.p:=center;
      cpSpaceAddBody(space, body);
   end else body:=StaticBody;

   inc(groupnum);
   if convex then checkshape:=False;
   if checkShape then begin
      if penMode=pmSegments then j:=length(verts)-2 else j:=length(verts)-1;
      for i :=0 to j do begin
         shape:= cpSegmentShapeNew(body, verts[i], verts[(i+1) mod poly.Nodes.count], 0.01);
         shape.e:=0.0; shape.u:= 0.5;
         shape.group:=groupnum;
         if body <> StaticBody then begin
            shape.collision_type:=1;
            cpSpaceAddShape(space, shape);
         end else begin
            cpSpaceAddStaticShape(space, shape);
         end;
         result:=shape;
      end;
   end else begin
      shape:= cpPolyShapeNew(body, length(verts), @verts[0], cpvzero);
      shape.e:=0.0; shape.u:= 0.5;
      if body <> StaticBody then shape.collision_type:=1;
      shape.group:=groupnum;
      cpSpaceAddShape(space, shape);
      result:=shape;
   end;
end;

procedure TForm1.EndSpline;
var i,c : integer;
    n,nt:TAffineVector;
    v : TVector;
begin
   if not assigned(Spline) then exit;
   if Spline.nodes.count<3 then begin // not enough points for poly
      FreeAndNil(spline);
      exit;
   end;
   case PenMode of
      pmSegments,pmPoly:  begin
        nt:=nullVector;
        for i:=0 to spline.nodes.count-3 do begin
           CalcPlaneNormal(spline.nodes[i].AsVector,spline.nodes[i+1].AsVector,spline.nodes[i+2].AsVector,n);
           AddVector(nt,n);
        end;
        NormalizeVector(nt);
        if VectorDotProduct(nt,ZVector)>0 then begin
           c:=spline.Nodes.count-1;
           for i:=0 to (c div 2) do begin
              v:=spline.Nodes[i].AsVector;
              spline.Nodes[i].AsVector:=spline.Nodes[c-i].AsVector;
              spline.Nodes[c-i].AsVector:=v;
           end;
           caption:='Counter Clockwise';
        end else begin
           caption:='Clockwise';
        end;
        spline.Position.SetPoint(0,0,0);
        AddPolyToSim(spline,true);
      end;
      pmChain,pmChainBreakable: AddChainToSim(Spline);
   end;
   FreeAndNil(spline);
end;

procedure TForm1.FormKeyPress(Sender: TObject; var Key: Char);
begin
   case upcase(key) of
      #32 : Pause:=not Pause;
      #27 : close;
      'S' : PenBody:=pbStatic;
      'D' : PenBody:=pbDynamic;

      '1' : PenMode:=pmPoly;
      '2' : PenMode:=pmSegments;
      '3' : PenMode:=pmChain;
      '4' : PenMode:=pmChainBreakable;
      '5' : PenMode:=pmCircle;
      '6' : PenMode:=pmMotor;
      '9' : PenMode:=pmPin;

      'M' : PenMode:=pmMove;

      'C' : ResetScene;

      '?' : GLHudText1.Visible:=not GLHudText1.Visible;
   end;
end;

procedure TForm1.GLDirectOpenGL1Render(Sender: TObject; var rci: TRenderContextInfo);
begin
   DrawPhysics(Sender,rci,space);
end;

procedure TForm1.FPSTimerTimer(Sender: TObject);
begin
   Caption:=format('PAR Munk Demo PM=%s PB=%s  %d bodies %d shapes %d arbiters %d constraints %s',[TPenModest[PenMode],TPenBodyst[PenBody],cpTotalbodies,cpTotalShapes,cpTotalArbiters, cpTotalConstraints, GLSceneViewer1.FramesPerSecondText(2)]);
   GLSceneViewer1.ResetPerformanceMonitor;
end;

procedure TForm1.GLSceneViewer1MouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
   GLCamera1.SceneScale:=GLCamera1.SceneScale+WheelDelta*0.0005;
   if GLCamera1.SceneScale<0.1 then GLCamera1.SceneScale:=0.1;
end;

end.
